{% extends "base.html" %}

{% from "macros.html" import pagination %}

{% block content %}
		<h1>Institutions index</h1>

		{{ pagination(paginator, 'institutions') }}
		
		<table class="table table-striped table-condensed">
			<thead>
				<tr>
					<th>Name</th>
				</tr>
			</thead>
			<tbody>
				{% for institution in objects %}
				<tr>
					<td><a href="{{ url_for('institution', institution_id=institution.id) }}">{{ institution.name }}</a></td>
				</tr>
				{% endfor %}
			</tbody>
		</table>

		{{ pagination(paginator, 'institutions') }}
{% endblock %}
