{% extends "base.html" %}

{% from "macros.html" import pagination %}

{%- block content %}
		<h1>Publications index</h1>

		{{ pagination(paginator, 'publications') }}

		<table class="table table-striped table-condensed">
			<thead>
				<tr>
					<th>Title</th>
					<th>Year</th>
				</tr>
			</thead>
			<tbody>
				{%- for publication in objects %}
				<tr>
					<td><a href="{{ url_for('publication', publication_id=publication.id) }}">{{ publication.title }}</a></td>
					<td>{{ publication.year }}</td>
				</tr>
				{%- endfor %}
			</tbody>
		</table>

		{{ pagination(paginator, 'publications') }}
{%- endblock %}
