// Generated by CoffeeScript 1.6.3
(function() {
  if (typeof String.prototype.startswith !== 'function') {
    String.prototype.startswith = function(str) {
      return this.slice(0, str.length) === str;
    };
  }

  (function($) {})(jQuery);

}).call(this);
